## Edits by Caroline
#
#	Added VMD callback-updated spinbox for MolID
# Add Seed also added an exclusion sphere - now fixed.
#	Rm Draw button (code is still present below - just commented out - just in case you want its functionality)
# Enabled 'Add Seed' only for the Sphere method
# Added Load cfg file button
# Created proc ::epock::openCFGfile which opens, parses, sends instructions for graphics creation based on file content, and closes CFG files
#		Edit 6th June: now compatible with both cfg saved from epock and /example/config.cfg
# Created proc ::epock::addSmth which encapsulates the functions previously done by add in/exclusion buttons. Edited to be compatible with input file content.
# 	Edit 6th June: included seeds
# In epock.tcl:
#		Added file-input compatible procs for each graphical representation.

## TO DO !!! ##
# - Do a tick box to choose between residue and atom contribution

#-----------------------------------------------------------------------
#
#	The epock
#
#	May 2013
#	Caroline Dahl, Benoist Laurent & Matthieu Chavent
#	Oxford University, LBT IBPC
#
#-----------------------------------------------------------------------

package provide epock 1.0
package require tile

namespace eval ::epock:: {
	
	variable target 	""
	variable target2 	""
	variable selection  "protein"
	variable cavity_name 	"cav"
	variable molid 		0
	variable radius 	5 	
	variable x_box	 	5	
	variable y_box 		5	
	variable z_box 		5	
	variable choice 1
	variable choice_method "sphere"
	variable list_graph_primitives ""
	variable listbox_sel 0	
	variable id_graph_draw 9999999
	variable exec_path      "epock"

	# options for configuration file	
	variable contiguous_choice "false"
	variable profile_choice "true"	
	variable contrib_choice "true"
	variable martini 0
	variable contiguous_cutoff 4.0
	variable grid_size 1.0
	variable padding 1.4

	#list ids treeview
	variable list_ids_treeview ""
	variable molid_volume_list ""
	
	# options for dynamic analysis
	variable fitting_choice "false"
	variable target_fitting 	""	
	variable trajectory_starting_point 0
	variable trajectory_ending_point -1
	variable trajectory_step 1
}

# Switch on molid tracer
trace add variable vmd_molecule write {::epock::molID_tracker;};

#package provide epock $::epock::version

#-----------------------------------------------------------------------
#
#	GUI
#
#-----------------------------------------------------------------------


#
# Set ::epock::exec_path to the appropriate value.
#
# If ::epock::exec_path exists, do nothing.
# Else, if it is a valid command (typically "epock"),
# ::epock::exec_path is converted to the absolute path to epock executable.
# Else, open a Tk window and ask the user to manually find Epock's executable.
#
proc ::epock::init_exec_path {} {
    if { [file exists $::epock::exec_path] == 0 } {
        set ::epock::exec_path [auto_execok $::epock::exec_path]
        if { [string length $::epock::exec_path] == 0 } {
            set ::epock::exec_path [tk_getOpenFile -title "Could not load exec. Please specify the path"]
        }
    }
}

proc ::epock::epockGUI {} {
	global env
	global auto_path
	variable dw
	
	if { [winfo exists .epock] } {
        	wm deiconify $dw
        	return
    	}
	
	set dw [toplevel ".epock"]
    	wm title $dw "Cavity volume analysis and visualization"	
	wm maxsize $dw 850 700  	
	frame $dw.top
    	wm protocol $dw WM_DELETE_WINDOW {
		::epock::quit
	}
    
	# Beginning of Menubar -----------------------
    	#$dw.menubar.help.menu add command -label "About" \
       	# -command {tk_messageBox -type ok -title "About The epock" \
                  #    -message "A tool for analysis of cavity volume.\nVersion $::epock::version\n\n(c) 2013 Paris and Oxford"}
    	#$dw.menubar.help.menu add command -label "Help..." \
        #-command "vmd_open_url http://matthieuchavent.com/"

    	## End of Menubar ----------------------------
    
    	## GUI features
    	## Make the notebook and set up Ctrl+Tab traversal    
    	ttk::notebook $dw.nb
    	pack $dw.nb
    	#ttk::notebook::enableTraversal $dw.nb 
    
    	## frame 1: -----------------------------------
    	ttk::frame $dw.nb.tab1
    	$dw.nb add $dw.nb.tab1 -text "Static analysis"

	# Message Box Target 1 for selection1
	message $dw.nb.tab1.target_msg -width 200 -text "Target1:" -padx 10
	entry $dw.nb.tab1.target -textvariable ::epock::target -background white \
		-selectbackground yellow2 -width 50

	# Message Box Target 2 for selection1
	message $dw.nb.tab1.target2_msg -width 200 -text "Target2:" -padx 10
	entry $dw.nb.tab1.target2 -textvariable ::epock::target2 -background white \
		-selectbackground yellow2 -width 50 

	# Message Box Target 2 for selection1
	message $dw.nb.tab1.selection_msg -width 200 -text "Selection:" -padx 10
	entry $dw.nb.tab1.selection -textvariable ::epock::selection -background white \
		-selectbackground yellow2 -width 50 


	# Message Box Cavity name to use the good name for config file and listbox
	message $dw.nb.tab1.cavity_name_msg -width 200 -text "Cavity Name:" -padx 10
	entry $dw.nb.tab1.cavity_name -textvariable ::epock::cavity_name -background white \
		-selectbackground yellow2 -width 50 

	# Molid to define on which molecule we will perform calculation
	message $dw.nb.tab1.molid_msg -width 60 -text "MolId:" -padx 2	
	set N_loaded_molecules [molinfo num]
	set ID_mols [molinfo list]
	set N_mols ""
	for {set molN 0} {$molN < $N_loaded_molecules} {incr molN } {
		lappend N_mols [lindex $ID_mols $molN]
	}
	spinbox $dw.nb.tab1.molid -values $N_mols -state readonly -width 3 -background white -selectbackground yellow2 -width 2 -command {
		set ::epock::molid [.epock.nb.tab1.molid get]
	}
	
	# Radius field used for sphere and cylinder calculation
	message $dw.nb.tab1.radius_msg -width 60 -text "Radius:" -padx 2
	entry $dw.nb.tab1.radius -textvariable ::epock::radius -background white -selectbackground yellow2 \
		-width 3 

	# X field used for box calculation		
	message $dw.nb.tab1.x_box_msg -width 60 -text "Box X: " -padx 2
	entry $dw.nb.tab1.x_box -textvariable ::epock::x_box -background white -selectbackground yellow2 \
		-width 3

	# Y field used for box calculation	
	message $dw.nb.tab1.y_box_msg -width 60 -text "Box Y:" -padx 2
	entry $dw.nb.tab1.y_box -textvariable ::epock::y_box -background white -selectbackground yellow2 \
		-width 3

	# Z field used for box calculation	
	message $dw.nb.tab1.z_box_msg -width 60 -text "Box Z:" -padx 2
	entry $dw.nb.tab1.z_box -textvariable ::epock::z_box -background white -selectbackground yellow2 \
		-width 3

	message $dw.nb.tab1.availAnalysis_msg  -width 60 -text "Analyses:" -padx 3

	# Radiobutton to choose sphere: enable radius and disable x_box, y_box, z_box and target2
	radiobutton $dw.nb.tab1.choice1 -text "Sphere Method" -variable ::epock::choice -value 1 -command {
		if {$::epock::choice == 1} {
			# enable
			.epock.nb.tab1.radius_msg configure -foreground black
			.epock.nb.tab1.radius configure -state normal
			.epock.nb.tab1.seed configure -state normal

			# disable
			.epock.nb.tab1.x_box_msg configure -foreground "gray60"
			.epock.nb.tab1.x_box configure -state disabled
			.epock.nb.tab1.y_box_msg configure -foreground "gray60"
			.epock.nb.tab1.y_box configure -state disabled
			.epock.nb.tab1.z_box_msg configure -foreground "gray60"
			.epock.nb.tab1.z_box configure -state disabled
			.epock.nb.tab1.target2_msg configure -foreground "gray60"
			.epock.nb.tab1.target2 configure -state disabled
			
			# choose the method name - used after
			set ::epock::choice_method "sphere"
		} {

		}		
	}

	# Radiobutton to choose box: enable  x_box, y_box, z_box and disable target2 and radius
	radiobutton $dw.nb.tab1.choice2 -text "Box Method" -variable ::epock::choice -value 2 -command {
		if {$::epock::choice == 2} {
			# enable
			.epock.nb.tab1.x_box_msg configure -foreground black
			.epock.nb.tab1.x_box configure -state normal
			.epock.nb.tab1.y_box_msg configure -foreground black
			.epock.nb.tab1.y_box configure -state normal
			.epock.nb.tab1.z_box_msg configure -foreground black
			.epock.nb.tab1.z_box configure -state normal
			.epock.nb.tab1.seed configure -state normal
			
			# disable
			.epock.nb.tab1.radius_msg configure -foreground "gray60"
			.epock.nb.tab1.radius configure -state disabled
			.epock.nb.tab1.target2_msg configure -foreground "gray60"
			.epock.nb.tab1.target2 configure -state disabled

			# choose the method name - used after			
			set ::epock::choice_method "box"
		} {

		}		
	}

	# Radiobutton to choose cylinder: enable target2 and radius and disable x_box, y_box, z_box 	
	radiobutton $dw.nb.tab1.choice3 -text "Cylinder Method" -variable ::epock::choice -value 3 -command {
		if {$::epock::choice == 3} {
			# enable
			.epock.nb.tab1.target2_msg configure -foreground black
			.epock.nb.tab1.target2 configure -state normal
			.epock.nb.tab1.radius_msg configure -foreground black
			.epock.nb.tab1.radius configure -state normal
			.epock.nb.tab1.seed configure -state normal

			# disable
			.epock.nb.tab1.x_box_msg configure -foreground "gray60"
			.epock.nb.tab1.x_box configure -state disabled
			.epock.nb.tab1.y_box_msg configure -foreground "gray60"
			.epock.nb.tab1.y_box configure -state disabled
			.epock.nb.tab1.z_box_msg configure -foreground "gray60"
			.epock.nb.tab1.z_box configure -state disabled

			# choose the method name - used after
			set ::epock::choice_method "cylinder"			
		} {
			# disable 			
			.epock.nb.tab1.x_box_msg configure -foreground "gray60"
			.epock.nb.tab1.x_box configure -state disabled
			.epock.nb.tab1.y_box_msg configure -foreground "gray60"
			.epock.nb.tab1.y_box configure -state disabled
			.epock.nb.tab1.z_box_msg configure -foreground "gray60"
			.epock.nb.tab1.z_box configure -state disabled
			.epock.nb.tab1.target2_msg configure -foreground "gray60"
			.epock.nb.tab1.target2 configure -state disabled
		}
	}
	

	#Add inclusion button: put the inclusion primitives in the listbox -----------
	button $dw.nb.tab1.inclusion -text "Add Inclusion" -background yellow -width 10 -command {
		::epock::addSmth 1 1 0 0 0 0 0 0 0
	}
	#End of Add inclusion button -------------------------------------------------

	#Add exclusion button: put the inclusion primitives in the listbox -----------
	button $dw.nb.tab1.exclusion -text "Add Exclusion" -background red -width 10 -command {
		::epock::addSmth 0 1 0 0 0 0 0 0 0
	}
	#End of Add exclusion button -------------------------------------------------

	#Seed button: put the seed primitives in the listbox -----------
	button $dw.nb.tab1.seed -text "Add Seed" -background cyan -width 10 -command {
		::epock::addSmth 0 1 0 0 0 0 0 0 1
	}
	#End Seedbutton: put the seed primitives in the listbox -----------

	#Remove button: remove primitives from the treeview -----------
	button $dw.nb.tab1.remove -text "Remove"  -width 10 -command {
	
		set nb_elt_selected [llength [.epock.nb.tab1.tree selection] ]
		
		# if several elements are selected in the treeview
		if {$nb_elt_selected > 1} {
		
		set elt_selected_list [.epock.nb.tab1.tree selection]
	
			foreach element $elt_selected_list {
			
			# if parent selection is empty remove all the children
			if {[.epock.nb.tab1.tree  parent $element] ==""} {
					
					#puts "children0:[.epock.nb.tab1.tree  children $element]"
					#puts "parent0:[.epock.nb.tab1.tree  parent $element]"			
			
					foreach children [.epock.nb.tab1.tree  children $element] {
						set graph_id_to_remove [lindex [ .epock.nb.tab1.tree item $children -values] 3]
						set molid_to_remove [lindex [ .epock.nb.tab1.tree item $children -values] 4] 
						
						if {$graph_id_to_remove !="" && $graph_id_to_remove !=""} {	
							graphics $molid_to_remove delete $graph_id_to_remove
						} 
					}
					.epock.nb.tab1.tree delete $element
					
			# if parent selection is not empty remove only the selected children		
			} else {

				if {[.epock.nb.tab1.tree  children $element] !="" } {

				} else {
						#puts "children2:[.epock.nb.tab1.tree  children $element]"
						#puts "parent2:[.epock.nb.tab1.tree  parent $element]"
						
						
						set graph_id_to_remove [lindex [ .epock.nb.tab1.tree item $element -values] 3]
						set molid_to_remove [lindex [ .epock.nb.tab1.tree item $element -values] 4] 
						
						#puts "$graph_id_to_remove $molid_to_remove"
						
						graphics $molid_to_remove delete $graph_id_to_remove
						.epock.nb.tab1.tree delete $element
				}
			
			}
			}							
										
		# if only one element is selected several elements are selected in the treeview	
		} else {	
			
			# if parent selection is empty remove all the children
			if {[.epock.nb.tab1.tree  parent [.epock.nb.tab1.tree selection]] ==""} {
					
					#puts "children0:[.epock.nb.tab1.tree  children [.epock.nb.tab1.tree selection]]"
					#puts "parent0:[.epock.nb.tab1.tree  parent [.epock.nb.tab1.tree selection]]"			
			
					foreach children [.epock.nb.tab1.tree  children [.epock.nb.tab1.tree selection]] {
						set graph_id_to_remove [lindex [ .epock.nb.tab1.tree item $children -values] 3]
						set molid_to_remove [lindex [ .epock.nb.tab1.tree item $children -values] 4] 
						
						if {$graph_id_to_remove !="" && $graph_id_to_remove !=""} {	
							graphics $molid_to_remove delete $graph_id_to_remove
						} 
					}
					.epock.nb.tab1.tree delete [.epock.nb.tab1.tree selection]
					
			# if parent selection is not empty remove only the selected children		
			} else {

				if {[.epock.nb.tab1.tree  children [.epock.nb.tab1.tree selection]] !="" } {

				} else {
						#puts "children2:[.epock.nb.tab1.tree  children [.epock.nb.tab1.tree selection]]"
						#puts "parent2:[.epock.nb.tab1.tree  parent [.epock.nb.tab1.tree selection]]"
						
						
						set graph_id_to_remove [lindex [ .epock.nb.tab1.tree item [.epock.nb.tab1.tree selection] -values] 3]
						set molid_to_remove [lindex [ .epock.nb.tab1.tree item [.epock.nb.tab1.tree selection] -values] 4] 
						
						puts "$graph_id_to_remove $molid_to_remove"
						
						graphics $molid_to_remove delete $graph_id_to_remove
						.epock.nb.tab1.tree delete [.epock.nb.tab1.tree selection]
				}
			
			}
		}
	}
	#End of Remove button -------------------------------------------------


	# create a treeview to store graphical primitives
	ttk::treeview $dw.nb.tab1.tree -columns "Shape Center Radius Id_primitive Molid Hidden Graph_prim"
	set ::epock::treeview $dw.nb.tab1.tree
	$dw.nb.tab1.tree column Shape -width 200 -anchor center 
	$dw.nb.tab1.tree heading Shape -text "Shape"
	$dw.nb.tab1.tree column Center -width 300 -anchor center
	$dw.nb.tab1.tree heading Center -text "Center"
	$dw.nb.tab1.tree column Radius -width 100 -anchor center
	$dw.nb.tab1.tree heading Radius -text "Radius/X,Y,Z"
	$dw.nb.tab1.tree column Id_primitive -width 50 -anchor center
	$dw.nb.tab1.tree heading Id_primitive -text "Id"
	$dw.nb.tab1.tree column Molid -width 50 -anchor center
	$dw.nb.tab1.tree heading Molid -text "MolId"
	$dw.nb.tab1.tree column Hidden -width 50 -anchor center
	$dw.nb.tab1.tree heading Hidden -text "Hidden"
	$dw.nb.tab1.tree column Graph_prim -width 50 -anchor center
	$dw.nb.tab1.tree heading Graph_prim -text "Graph_prim"
	
	# create a tag to highlight 
	$dw.nb.tab1.tree tag configure "hidden" -foreground red

	# hide/show graphical primitives when double-click on a treview line
	bind  .epock.nb.tab1.tree <Double-Button-1> {
		
		set graph_id [lindex [ .epock.nb.tab1.tree item [.epock.nb.tab1.tree selection] -values] 3]
		set molid_id [lindex [ .epock.nb.tab1.tree item [.epock.nb.tab1.tree selection] -values] 4]
		set hidden_tag [lindex [ .epock.nb.tab1.tree item [.epock.nb.tab1.tree selection] -values] 5] 		
		
		if {$graph_id != ""} {

		#.epock.nb.tab1.tree set [.epock.nb.tab1.tree selection] Hidden 0
		::epock::hide_graphic_primitive $graph_id $molid_id $hidden_tag [.epock.nb.tab1.tree selection] 

		if {$hidden_tag == 0} {
			.epock.nb.tab1.tree item [.epock.nb.tab1.tree selection]  -tags "hidden"	
			.epock.nb.tab1.tree set  [.epock.nb.tab1.tree selection]  Hidden 1
		} else {
			.epock.nb.tab1.tree item [.epock.nb.tab1.tree selection]  -tags "visible"	
			.epock.nb.tab1.tree set  [.epock.nb.tab1.tree selection]  Hidden 0		
		}

		} else {
			#.epock.nb.tab1.tree set [.epock.nb.tab1.tree selection] Hidden 0
			set parent_name [.epock.nb.tab1.tree  children [.epock.nb.tab1.tree selection]]
			
			foreach children $parent_name {
			
				set graph_id [lindex [ .epock.nb.tab1.tree item $children -values] 3]
				set molid_id [lindex [ .epock.nb.tab1.tree item $children -values] 4]
				set hidden_tag_children [lindex [ .epock.nb.tab1.tree item $children -values] 5]
					
				#.epock.nb.tab1.tree set $children Hidden 0
				#::epock::hide_graphic_primitive $graph_id $molid_id $hidden_tag $children

				if {$hidden_tag == 0 && $hidden_tag_children == $hidden_tag} {
					::epock::hide_graphic_primitive $graph_id $molid_id $hidden_tag $children
					.epock.nb.tab1.tree item $children -tags "hidden"	
					.epock.nb.tab1.tree set  $children Hidden 1
				} elseif {$hidden_tag == 0 && $hidden_tag_children != $hidden_tag} {
					.epock.nb.tab1.tree item $children -tags "hidden"	
					.epock.nb.tab1.tree set  $children Hidden 1			
				} elseif { $hidden_tag == 1 && $hidden_tag_children == $hidden_tag} {
					::epock::hide_graphic_primitive $graph_id $molid_id $hidden_tag $children
					.epock.nb.tab1.tree item $children -tags "visible"	
					.epock.nb.tab1.tree set  $children Hidden 0
				} elseif { $hidden_tag == 1 && $hidden_tag_children != $hidden_tag} {
					.epock.nb.tab1.tree item $children -tags "visible"
					.epock.nb.tab1.tree set  $children Hidden 0				
				}

			}			
	
			if {$hidden_tag == 0} {
				.epock.nb.tab1.tree item [.epock.nb.tab1.tree selection] -tags "hidden"	
				.epock.nb.tab1.tree set  [.epock.nb.tab1.tree selection] Hidden 1
			} else {
				.epock.nb.tab1.tree item [.epock.nb.tab1.tree selection] -tags "visible"	
				.epock.nb.tab1.tree set  [.epock.nb.tab1.tree selection] Hidden 0
			}
			
		}

	#puts "molid: $molid_id" 
	#puts "graphid: $graph_id" 	
	break
	}

	# options for configuration file -----------
	message $dw.nb.tab1.configuration_opts -width 150 -text "Configuration options:" -padx 2
	message $dw.nb.tab1.contiguous_msg -width 100 -text "Contiguous opt.:" -padx 2
	radiobutton $dw.nb.tab1.contiguous_choice1 -text "true" -variable ::epock::contiguous_choice -value "true"	
	radiobutton $dw.nb.tab1.contiguous_choice2 -text "false" -variable ::epock::contiguous_choice -value "false"	

	message $dw.nb.tab1.profile_msg -width 100 -text "Profile opt.:" -padx 2
	radiobutton $dw.nb.tab1.profile_choice1 -text "true" -variable ::epock::profile_choice -value "true"	
	radiobutton $dw.nb.tab1.profile_choice2 -text "false" -variable ::epock::profile_choice -value "false"

	message $dw.nb.tab1.contrib_msg -width 100 -text "Contrib. opt.:" -padx 2
	radiobutton $dw.nb.tab1.contrib_choice1 -text "true" -variable ::epock::contrib_choice -value "true"	
	radiobutton $dw.nb.tab1.contrib_choice2 -text "false" -variable ::epock::contrib_choice -value "false"

	message $dw.nb.tab1.martini_msg -width 100 -text "MARTINI ff:" -padx 2
	checkbutton $dw.nb.tab1.martini -variable ::epock::martini
	
	message $dw.nb.tab1.gridres_msg -width 100 -text "Grid res.:" -padx 2
	entry $dw.nb.tab1.gridres -textvariable ::epock::grid_size -background white -selectbackground yellow2 \
		-width 3 

	message $dw.nb.tab1.contigcut_msg -width 100 -text "Contig. cut-off:" -padx 2
	entry $dw.nb.tab1.contigcut -textvariable ::epock::contiguous_cutoff -background white -selectbackground yellow2 \
		-width 3 

	message $dw.nb.tab1.padding_msg -width 100 -text "Grid Sph. Size:" -padx 2
	entry $dw.nb.tab1.padding -textvariable ::epock::padding -background white -selectbackground yellow2 \
		-width 3 

	# end of options for configuration file -----------

	# button to save the config file
	button $dw.nb.tab1.save_cfg -text "save cfg."  -width 10 -command {
		set filename [tk_getSaveFile]
		
		# check if the user has the permission to write file in the directory
		# if yes save the file otherwise popup a message
		if {[file writable [file dirname $filename]]==0} {	
			tk_messageBox -message "You have not the permission to write in this directory, please change your path" -type ok
		} else {
			set fp [open $filename "w"]		
			::epock::make_cfg $fp
		}	
	}


	# Launch epock calculation	
	button $dw.nb.tab1.calculation -text "calculate Volume" -background white -width 20 -command {
		
		# create the config file but first check the permission rigths
		if {[file writable "."]!=0} {	
			set fp [open "./config_temp.cfg" "w"]	
			::epock::make_cfg $fp
				
			# create temp. files
			set pdb_sel_all [atomselect $::epock::molid $::epock::selection]
			set pdb_filename "./temp_$::epock::choice_method.pdb"
			$pdb_sel_all writepdb $pdb_filename
			set config_filename "./config_temp.cfg"
			set output "./volume.dat"
			set name $::epock::choice_method
			::epock::init_exec_path
			::epock::launch_epock $::epock::molid $pdb_filename $config_filename $output ".epock.nb.tab1.tree" $::epock::exec_path $::epock::martini
			::epock::epock_results_GUI $output $::epock::molid ".epock.nb.tab1.tree"
        } else {
			tk_messageBox -message "You have not the permission to write in this directory, please change your path" -type ok
		}	
	}
	
	button $dw.nb.tab1.readCFG -text "Load cfg" -background green -width 6 -command {
		::epock::openCFGfile 
		#puts "OUTPUT> Graphing. (Link up to cpp script!)\n"
	}

# ----------------------------------------------------------------------------
#
#   set the position of the different fields and buttons on the GUI for tab1
#
# ----------------------------------------------------------------------------

	set row 1
	#pack $dw.nb.tab1 -side top -anchor w
    #grid $dw.nb.tab1.welcome -row $row -column 0 -columnspan 4 -padx 5; #-sticky nsew
        
	# position molid field
	incr row 
    grid $dw.nb.tab1.molid_msg -row $row -column 0 -sticky w -padx 20
	grid $dw.nb.tab1.molid -row $row -column 0 -columnspan 1 -sticky w -pady 5 -padx 100
	grid $dw.nb.tab1.configuration_opts -row $row -column 0 -sticky w -padx 550

	# position target1 field + contiguous choice
    incr row
    grid $dw.nb.tab1.target_msg -row $row -column 0 -sticky w -padx 10
	grid $dw.nb.tab1.target -row $row -column 0 -columnspan 1 -sticky w -pady 5  -padx 100
	grid $dw.nb.tab1.contiguous_msg -row $row -column 0 -sticky w -padx 500
	grid $dw.nb.tab1.contiguous_choice1  -row $row -column 0 -sticky w -padx 600
	grid $dw.nb.tab1.contiguous_choice2  -row $row -column 0 -sticky w -padx 650

	# position target2 field + profile choice 
    incr row
    grid $dw.nb.tab1.target2_msg -row $row -column 0 -sticky w -padx 10
	grid $dw.nb.tab1.target2 -row $row -column 0 -columnspan 1 -sticky w -pady 5  -padx 100
	grid $dw.nb.tab1.profile_msg -row $row -column 0 -sticky w -padx 500
	grid $dw.nb.tab1.profile_choice1  -row $row -column 0 -sticky w -padx 600
	grid $dw.nb.tab1.profile_choice2  -row $row -column 0 -sticky w -padx 650	
	# put it in gray and disable at the beginning
	$dw.nb.tab1.target2_msg configure -foreground "gray60"
	$dw.nb.tab1.target2 configure -state disabled

	# position selection field + contrib choice
	incr row
	grid $dw.nb.tab1.selection_msg -row $row -column 0 -sticky w -padx 10
	grid $dw.nb.tab1.selection -row $row -column 0 -columnspan 1 -sticky w -pady 5 -padx 100 
	grid $dw.nb.tab1.contrib_choice1  -row $row -column 0 -sticky w -padx 600		
	grid $dw.nb.tab1.contrib_msg -row $row -column 0 -sticky w -padx 500
	grid $dw.nb.tab1.contrib_choice1  -row $row -column 0 -sticky w -padx 600
	grid $dw.nb.tab1.contrib_choice2  -row $row -column 0 -sticky w -padx 650
	
	# position cavity name field + martini option
	incr row
    	grid $dw.nb.tab1.cavity_name_msg -row $row -column 0 -sticky w 
	grid $dw.nb.tab1.cavity_name -row $row -column 0 -columnspan 1 -sticky w -pady 5 -padx 100 -pady 10
	grid $dw.nb.tab1.martini_msg -row $row -column 0 -sticky w -padx 500
	grid $dw.nb.tab1.martini -row $row -column 0 -sticky w -padx 600

	# position sphere tick box and radius field + resolution of the grid 
	incr row
	grid $dw.nb.tab1.availAnalysis_msg -row $row -column 0 -columnspan 1 -sticky w -padx 5 
	grid $dw.nb.tab1.choice1 -row $row -column 0 -columnspan 1  -sticky w -padx 90
	grid $dw.nb.tab1.radius_msg -row $row -column 0 -columnspan 1 -sticky w -padx 240
	grid $dw.nb.tab1.radius -row $row -column 0 -columnspan 1   -sticky w -padx 290

	grid $dw.nb.tab1.gridres_msg -row $row -column 0 -sticky w -padx 570
	grid $dw.nb.tab1.gridres -row $row -column 0 -columnspan 1 -sticky w -pady 5 -padx 670

	# position cylinder tick box + contiguous cutoff		
	incr row
	grid $dw.nb.tab1.choice3 -row $row -column 0 -columnspan 1 -sticky w -padx 90
	grid $dw.nb.tab1.contigcut_msg -row $row -column 0 -sticky w -padx 570
	grid $dw.nb.tab1.contigcut -row $row -column 0 -columnspan 1 -sticky w -pady 5 -padx 670

	# position box tick box and x,y,z box fields + padding
	incr row
	grid $dw.nb.tab1.choice2 -row $row -column 0 -columnspan 1 -sticky w -padx 90
	grid $dw.nb.tab1.x_box_msg -row $row -column 0 -columnspan 1 -sticky w  -padx 190
	grid $dw.nb.tab1.x_box -row $row -column 0 -columnspan 1 -sticky w   -padx 230
	grid $dw.nb.tab1.y_box_msg -row $row -column 0 -columnspan 1 -sticky w   -padx 270
	grid $dw.nb.tab1.y_box -row $row -column 0 -columnspan 1 -sticky w   -padx 315
	grid $dw.nb.tab1.z_box_msg -row $row -column 0 -columnspan 1  -sticky w  -padx 345
	grid $dw.nb.tab1.z_box -row $row -column 0 -columnspan 1 -sticky w -padx 395
	# disable fields at the beginning
	$dw.nb.tab1.x_box_msg configure -foreground "gray60"
	$dw.nb.tab1.x_box configure -state disabled
	$dw.nb.tab1.y_box_msg configure -foreground "gray60"
	$dw.nb.tab1.y_box configure -state disabled
	$dw.nb.tab1.z_box_msg configure -foreground "gray60"
	$dw.nb.tab1.z_box configure -state disabled	
	grid $dw.nb.tab1.padding_msg -row $row -column 0 -sticky w -padx 570
	grid $dw.nb.tab1.padding -row $row -column 0 -columnspan 1 -sticky w -pady 5 -padx 670

	
	# position the buttons Draw, Inclusion/exclusion spheres, remove
	incr row
	#grid $dw.nb.tab1.graph -row $row -column 0 -sticky w -padx 10  -pady 10
	grid $dw.nb.tab1.inclusion -row $row -column 0 -sticky w -padx 120 -pady 10	
	grid $dw.nb.tab1.exclusion -row $row -column 0 -sticky w -padx 230 -pady 10
	grid $dw.nb.tab1.seed -row $row -column 0 -sticky w -padx 340 -pady 10	
	grid $dw.nb.tab1.remove -row $row -column 0 -sticky w -padx 450 -pady 10	

	# position the treeview for the configuration file
	incr row
	grid $dw.nb.tab1.tree -row  $row -column 0 -columnspan 2 -sticky w

	# position buttons Calculate, Save cfg., Read cfg.
	incr row
	grid $dw.nb.tab1.calculation -row $row -column 0 -sticky w -padx 10  -pady 10	
	grid $dw.nb.tab1.save_cfg  -row $row -column 0 -sticky w -padx 300 -pady 10
	grid $dw.nb.tab1.readCFG -row $row -column 0 -sticky w -padx 500 -pady 10
	#grid $dw.nb.tab1.calculation -row $row -column 1 -columnspan 1 -sticky e  -pady 10 -padx 6
	
	#grid columnconfigure $dw.nb.tab1 2 -maxsize 300;
	
	## End of frame 1: -----------------------------------	
	
	
	## frame 2: ------------------------------------------
    ttk::frame $dw.nb.tab2
    $dw.nb add $dw.nb.tab2 -text "Dynamic analysis"
    
    # options for fitting -----------
	message $dw.nb.tab2.fitting_traj -width 150 -text "Trajectory fitting:" -padx 2
	radiobutton $dw.nb.tab2.fitting_choice1 -text "true" -variable ::epock::fitting_choice -value "true" -command {
			# enable
			.epock.nb.tab2.target_fitting_msg configure -foreground black
			.epock.nb.tab2.target_fitting configure -state normal
	}	
	radiobutton $dw.nb.tab2.fitting_choice2 -text "false" -variable ::epock::fitting_choice -value "false" -command {
			# disable 			
			.epock.nb.tab2.target_fitting_msg configure -foreground "gray60"
			.epock.nb.tab2.target_fitting configure -state disabled
	}
	
	# Message Box fitting choice
	message $dw.nb.tab2.target_fitting_msg -width 200 -text "Target for fit:" -padx 10
	entry $dw.nb.tab2.target_fitting -textvariable ::epock::target_fitting -background white \
		-selectbackground yellow2 -width 50 
	# disable fields at the beginning
	$dw.nb.tab2.target_fitting_msg configure -foreground "gray60"
	$dw.nb.tab2.target_fitting configure -state disabled
	
	# Message Box trajectory options
	message $dw.nb.tab2.trajectory_msg -width 200 -text "Trajectory options:" -padx 10	
	message $dw.nb.tab2.trajectory_starting_point_msg -width 50 -text "From:" -padx 10 
	entry $dw.nb.tab2.trajectory_starting_point -textvariable ::epock::trajectory_starting_point -background white -selectbackground yellow2 -width 3	
	message $dw.nb.tab2.trajectory_ending_point_msg -width 50 -text "To:" -padx 10
	#set ::epock::trajectory_ending_point [expr [molinfo $::epock::molid get numframes] - 1]
	entry $dw.nb.tab2.trajectory_ending_point -textvariable ::epock::trajectory_ending_point -background white -selectbackground yellow2 -width 3	
	message $dw.nb.tab2.trajectory_step_msg -width 50 -text "Step:" -padx 10
	entry $dw.nb.tab2.trajectory_step_point -textvariable ::epock::trajectory_step -background white -selectbackground yellow2 -width 3
		
	# Launch epock calculation	for dynamic results
	button $dw.nb.tab2.calculation -text "calculate Volume" -background white -width 20 -command {
		
		# Remove temporary files:
		file delete "./config_temp.cfg"

		
		# create the config file but first check the permission rights	
		if {[file writable "."]!=0} {	
			set fp [open "./config_temp.cfg" "w"]	
			::epock::make_cfg $fp
	
			# create temp. files
			set pdb_sel_all [atomselect $::epock::molid $::epock::selection]
			set pdb_filename "./temp_$::epock::choice_method.pdb"
			$pdb_sel_all writepdb $pdb_filename
			set config_filename "./config_temp.cfg"
			set output "./volume.dat"
			set name $::epock::choice_method
			
			# TO DO: check if xtc file available if possible 
			# molinfo $::epock::molid get filetype
			
			# fit the trajectory
			if {$::epock::fitting_choice == true} {
				set ref [atomselect $::epock::molid $::epock::target_fitting frame $::epock::trajectory_starting_point]
				set all [atomselect $::epock::molid "all"]

				#set nframes [molinfo $::epock::molid get numframes]
					for {set i $::epock::trajectory_starting_point} {$i < $::epock::trajectory_ending_point} {incr 1} {
						$sel frame $i
						$all frame $i
						$all move [measure fit $sel $ref]
					}
			}
			
			# Create a pdb files from the trajectory
			set trajectory_filename "./temp_trajectory.trr"
			animate write trr $trajectory_filename beg $::epock::trajectory_starting_point end $::epock::trajectory_ending_point skip $::epock::trajectory_step sel [atomselect $::epock::molid $::epock::selection] $::epock::molid

			
			# launch the function from the epock.tcl file and first check if the path is good
			# if not launch a window to give the path
			::epock::init_exec_path
			::epock::launch_epock_dynamic $::epock::molid $pdb_filename $trajectory_filename $config_filename $output ".epock.nb.tab1.tree" $::epock::exec_path $::epock::martini
			::epock::epock_results_dynamic_GUI $output $::epock::molid ".epock.nb.tab1.tree"
		} else {
			tk_messageBox -message "You have not the permission to write in this directory, please change your path" -type ok
		}				
	}	

# ----------------------------------------------------------------------------
#
#   set the position of the different fields and buttons on the GUI for tab2
#
# ----------------------------------------------------------------------------

	set row 1
	incr row 
	incr row
	        
	# position molid field
	incr row 
    grid $dw.nb.tab2.fitting_traj -row $row -column 0 -sticky w -padx 20
	grid $dw.nb.tab2.fitting_choice1 -row $row -column 0 -sticky w -padx 200
	grid $dw.nb.tab2.fitting_choice2  -row $row -column 0 -sticky w -padx 260
	
	# position trajectory fitting
    incr row
    grid $dw.nb.tab2.target_fitting_msg -row $row -column 0 -sticky w -padx 10
	grid $dw.nb.tab2.target_fitting -row $row -column 0 -columnspan 1 -sticky w -pady 5  -padx 100

	# position trajectory options
	incr row
    incr row
    grid $dw.nb.tab2.trajectory_msg -row $row -column 0 -sticky w -padx 10
    incr row
	grid $dw.nb.tab2.trajectory_starting_point_msg -row $row -column 0 -columnspan 1 -sticky w -padx 5 
	grid $dw.nb.tab2.trajectory_starting_point -row $row -column 0 -columnspan 1  -sticky w -padx 50
	grid $dw.nb.tab2.trajectory_ending_point_msg -row $row -column 0 -columnspan 1 -sticky w -padx 200
	grid $dw.nb.tab2.trajectory_ending_point -row $row -column 0 -columnspan 1   -sticky w -padx 245
	grid $dw.nb.tab2.trajectory_step_msg -row $row -column 0 -columnspan 1 -sticky w -padx 300
	grid $dw.nb.tab2.trajectory_step_point -row $row -column 0 -columnspan 1   -sticky w -padx 345
	

	# position "calculate volume" button
    incr row
    grid  $dw.nb.tab2.calculation -row $row -column 0 -sticky w -padx 120 -pady 10	
  
}

# ::epock::molID_tracker ---------------------------------------------------------
#    Called whenever a molecule is loaded or deleted,
#    to update the molecule ID spinbox 
#		 Should ideally also call relevant functions to erase graphics for deleted molecules.
# ------------------------------------------------------------------------------

proc ::epock::molID_tracker {} {
	set N_loaded_molecules [molinfo num]
	
	if {$N_loaded_molecules == 0} {
		.epock.nb.tab1.molid configure -values "";
		# Ensure that the molID field is cleared.
		.epock.nb.tab1.molid set ""; 

	} else {
		# Enable Rep0 button
		set pre_exists 0
		set temp_proteinID [.epock.nb.tab1.molid get]
		set ID_mols [molinfo list];
		set N_mols ""
		for {set molN 0} {$molN < $N_loaded_molecules} {incr molN } {
			if {[lindex $ID_mols $molN] == $temp_proteinID} {
				set pre_exists 1
			}
			lappend N_mols [lindex $ID_mols $molN]
		}
		# Add new molecule IDs to the spinbox
		.epock.nb.tab1.molid configure -values $N_mols
		
		#### If the proteinID that the user last worked on still exists, set the molID field to this.
		if {$pre_exists == 1} {
			.epock.nb.tab1.molid set $temp_proteinID
		}
	}
	#To-do: Erase graphics for deleted molcules, where present.
}


proc ::epock::make_cfg {fp} {

# write parameters in a configfile
	puts $fp "\[DEFAULT\]"
	puts $fp "grid_spacing = $::epock::grid_size"
	puts $fp "contiguous = $::epock::contiguous_choice"
	puts $fp "padding = $::epock::padding"
   	if {$::epock::contiguous_choice == "true"} {	
		puts $fp "contiguous_cutoff = $::epock::contiguous_cutoff"
   	}	
	puts $fp "profile = $::epock::profile_choice"

 	if {$::epock::contrib_choice == "true"} {
		puts $fp "contribution = residue"
	}

puts $fp ""
	
foreach lvl1 [.epock.nb.tab1.tree children {}] {
	puts $fp "\[$lvl1\]"
	#puts $fp "[lindex [ .epock.nb.tab1.tree item  $lvl1 -values] 0] = [lindex [ .epock.nb.tab1.tree item  $lvl1 -values] 1] [lindex [ .epock.nb.tab1.tree item  $lvl1 -values] 2]"
	foreach lvl2 [.epock.nb.tab1.tree children $lvl1] { 
		puts $fp "[lindex [ .epock.nb.tab1.tree item  $lvl2 -values] 0] = [lindex [ .epock.nb.tab1.tree item  $lvl2 -values] 1] [lindex [ .epock.nb.tab1.tree item  $lvl2 -values] 2]"
	}
	puts $fp ""
}

close $fp
}


#-------------------------------------------------------------
#
#		Add in/exclusion, by Button or via CFG file
#
#-------------------------------------------------------------
proc ::epock::addSmth {inclusion byButton x y z x2 y2 z2 seed} {
	
	# little hack to overwrite molid number...
        set ::epock::molid [lindex [molinfo list]]

	if {$seed} {
			if {$::epock::choice_method == "sphere"} {		
				if {$byButton} {
					set center [measure center [atomselect $::epock::molid $::epock::target]]
					set id_graph_prim [::epock::display_maximum_inclusion_volume_sphere $::epock::target $::epock::molid $::epock::grid_size $::epock::radius "cyan"]
				} else {
					set center [list $x $y $z];
					set id_graph_prim [::epock::display_maximum_inclusion_volume_sphere_byFile $center $::epock::molid $::epock::grid_size $::epock::radius "cyan"];
				}
				if { [.epock.nb.tab1.tree exists $::epock::cavity_name] == 1} {
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "contiguous_seed_sphere"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				} else {
					.epock.nb.tab1.tree insert {} end -id $::epock::cavity_name -text "$::epock::cavity_name"
					.epock.nb.tab1.tree set  $::epock::cavity_name Shape ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Center ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Radius ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Id_primitive ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Molid ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Hidden ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Graph_prim ""
			
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "contiguous_seed_sphere"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				}
			}
			
			if {$::epock::choice_method == "cylinder"} {
				if {$byButton} {
					set center1 [measure center [atomselect $::epock::molid $::epock::target]]
					set center2 [measure center [atomselect $::epock::molid $::epock::target2]]

					#create graphical primitive
					# launch the function ::epock::display_maximum_inclusion_volume_cylinder from the epock.tcl file			
					set id_graph_prim [::epock::display_maximum_inclusion_volume_cylinder $::epock::target $::epock::target2 $::epock::molid $::epock::grid_size $::epock::radius "cyan"]			
				} else {
					set center1 [list $x $y $z];
					set center2 [list $x2 $y2 $z2]; 
					set id_graph_prim [::epock::display_maximum_inclusion_volume_cylinder_byFile $center1 $center2 $::epock::molid $::epock::grid_size $::epock::radius "cyan"]		
				}

				# store infos in the treeview
				if { [.epock.nb.tab1.tree exists $::epock::cavity_name] == 1} {
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "contiguous_seed_cylinder"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center1 0]] [format %0.2f [lindex $center1 1]] [format %0.2f [lindex $center1 2]]   [format %0.2f [lindex $center2 0]] [format %0.2f [lindex $center2 1]] [format %0.2f [lindex $center2 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				} else {
					.epock.nb.tab1.tree insert {} end -id $::epock::cavity_name -text "$::epock::cavity_name"
					.epock.nb.tab1.tree set  $::epock::cavity_name Shape ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Center ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Radius ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Id_primitive ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Molid ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Hidden 0
					.epock.nb.tab1.tree set  $::epock::cavity_name Graph_prim ""

					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "contiguous_seed_cylinder"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center1 0]] [format %0.2f [lindex $center1 1]] [format %0.2f [lindex $center1 2]]   [format %0.2f [lindex $center2 0]] [format %0.2f [lindex $center2 1]] [format %0.2f [lindex $center2 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				}
			}
			if {$::epock::choice_method == "box"} {
				if {$byButton} {
					set center [measure center [atomselect $::epock::molid $::epock::target]]

					#create graphical primitive
					# launch the function ::epock::display_maximum_inclusion_volume_box from the epock.tcl file
					set id_graph_prim [::epock::display_maximum_inclusion_volume_box $::epock::target $::epock::x_box $::epock::y_box $::epock::z_box  $::epock::molid $::epock::grid_size "cyan"]
				} else {
					set center [list $x $y $z]
					set id_graph_prim [::epock::display_maximum_inclusion_volume_box_byFile $center $::epock::x_box $::epock::y_box $::epock::z_box  $::epock::molid $::epock::grid_size "cyan"]
				}

				# store infos in the treeview
				if { [.epock.nb.tab1.tree exists $::epock::cavity_name] == 1} {
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "contiguous_seed_box"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::x_box $::epock::y_box $::epock::z_box"
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim   
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				} else {
					.epock.nb.tab1.tree insert {} end -id $::epock::cavity_name -text "$::epock::cavity_name"
					.epock.nb.tab1.tree set  $::epock::cavity_name Shape ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Center ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Radius ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Id_primitive ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Molid ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Hidden 0
					.epock.nb.tab1.tree set  $::epock::cavity_name Graph_prim ""

					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "contiguous_seed_box"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::x_box $::epock::y_box $::epock::z_box"
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim   
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				}			
			}		
	} else {
	
		## In/exclusion?
		if {$inclusion} {
			if {$::epock::choice_method == "sphere"} {
				if {$byButton} {
					set center [measure center [atomselect $::epock::molid $::epock::target]]
					#create graphical primitive
					# launch the function ::epock::display_maximum_inclusion_volume_sphere from the epock.tcl file
					set id_graph_prim [::epock::display_maximum_inclusion_volume_sphere $::epock::target $::epock::molid $::epock::grid_size $::epock::radius "yellow"]
				} else {
					set center [list $x $y $z]; 
					set id_graph_prim [::epock::display_maximum_inclusion_volume_sphere_byFile $center $::epock::molid $::epock::grid_size $::epock::radius "yellow"];
				}
				
				# store infos in the treeview			
				if { [.epock.nb.tab1.tree exists $::epock::cavity_name] == 1} {
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "include_sphere"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				} else {
					.epock.nb.tab1.tree insert {} end -id $::epock::cavity_name -text "$::epock::cavity_name"
					.epock.nb.tab1.tree set  $::epock::cavity_name Shape ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Center ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Radius ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Id_primitive ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Molid ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Hidden 0
					.epock.nb.tab1.tree set  $::epock::cavity_name Graph_prim ""

					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "include_sphere"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				}
			}

			if {$::epock::choice_method == "cylinder"} {
				if {$byButton} {
					set center1 [measure center [atomselect $::epock::molid $::epock::target]]
					set center2 [measure center [atomselect $::epock::molid $::epock::target2]]

					#create graphical primitive
					# launch the function ::epock::display_maximum_inclusion_volume_cylinder from the epock.tcl file			
					set id_graph_prim [::epock::display_maximum_inclusion_volume_cylinder $::epock::target $::epock::target2 $::epock::molid $::epock::grid_size $::epock::radius "yellow"]			
				} else {
					set center1 [list $x $y $z];
					set center2 [list $x2 $y2 $z2]; 
					set id_graph_prim [::epock::display_maximum_inclusion_volume_cylinder_byFile $center1 $center2 $::epock::molid $::epock::grid_size $::epock::radius "yellow"]		
				}

				# store infos in the treeview
				if { [.epock.nb.tab1.tree exists $::epock::cavity_name] == 1} {
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "include_cylinder"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center1 0]] [format %0.2f [lindex $center1 1]] [format %0.2f [lindex $center1 2]]   [format %0.2f [lindex $center2 0]] [format %0.2f [lindex $center2 1]] [format %0.2f [lindex $center2 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				} else {
					.epock.nb.tab1.tree insert {} end -id $::epock::cavity_name -text "$::epock::cavity_name"
					.epock.nb.tab1.tree set  $::epock::cavity_name Shape ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Center ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Radius ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Id_primitive ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Molid ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Hidden 0
					.epock.nb.tab1.tree set  $::epock::cavity_name Graph_prim ""

					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "include_cylinder"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center1 0]] [format %0.2f [lindex $center1 1]] [format %0.2f [lindex $center1 2]]   [format %0.2f [lindex $center2 0]] [format %0.2f [lindex $center2 1]] [format %0.2f [lindex $center2 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				}
			}
			
			if {$::epock::choice_method == "box"} {
				if {$byButton} {
					set center [measure center [atomselect $::epock::molid $::epock::target]]

					#create graphical primitive
					# launch the function ::epock::display_maximum_inclusion_volume_box from the epock.tcl file
					set id_graph_prim [::epock::display_maximum_inclusion_volume_box $::epock::target $::epock::x_box $::epock::y_box $::epock::z_box  $::epock::molid $::epock::grid_size "yellow"]
				} else {
					set center [list $x $y $z]
					set id_graph_prim [::epock::display_maximum_inclusion_volume_box_byFile $center $::epock::x_box $::epock::y_box $::epock::z_box  $::epock::molid $::epock::grid_size "yellow"]
				}

				# store infos in the treeview
				if { [.epock.nb.tab1.tree exists $::epock::cavity_name] == 1} {
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "include_box"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::x_box $::epock::y_box $::epock::z_box"
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim   
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				} else {
					.epock.nb.tab1.tree insert {} end -id $::epock::cavity_name -text "$::epock::cavity_name"
					.epock.nb.tab1.tree set  $::epock::cavity_name Shape ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Center ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Radius ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Id_primitive ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Molid ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Hidden 0
					.epock.nb.tab1.tree set  $::epock::cavity_name Graph_prim ""

					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "include_box"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::x_box $::epock::y_box $::epock::z_box"
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim   
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				}
			}
		} else {
			## Exclusion
			if {$::epock::choice_method == "sphere"} {
				if {$byButton} {
					set center [measure center [atomselect $::epock::molid $::epock::target]]
					# launch the function ::epock::display_maximum_inclusion_volume_sphere from the epock.tcl file
					set id_graph_prim [::epock::display_maximum_inclusion_volume_sphere $::epock::target $::epock::molid $::epock::grid_size $::epock::radius "red"]
				} else {
					set center [list $x $y $z]
					set id_graph_prim [::epock::display_maximum_inclusion_volume_sphere_byFile $center $::epock::molid $::epock::grid_size $::epock::radius "red"]
				}
				
				if { [.epock.nb.tab1.tree exists $::epock::cavity_name] == 1} {
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "exclude_sphere"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				} else {
					.epock.nb.tab1.tree insert {} end -id $::epock::cavity_name -text "$::epock::cavity_name"
					.epock.nb.tab1.tree set  $::epock::cavity_name Shape ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Center ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Radius ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Id_primitive ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Molid ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Hidden 0
					.epock.nb.tab1.tree set  $::epock::cavity_name Graph_prim ""

					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "exclude_sphere"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				}
			}

			if {$::epock::choice_method == "cylinder"} {
				if {$byButton} {
					set center1 [measure center [atomselect $::epock::molid $::epock::target]]
					set center2 [measure center [atomselect $::epock::molid $::epock::target2]]
					# launch the function ::epock::display_maximum_inclusion_volume_cylinder from the epock.tcl file
					set id_graph_prim [::epock::display_maximum_inclusion_volume_cylinder $::epock::target $::epock::target2 $::epock::molid $::epock::grid_size $::epock::radius "red"]
				}	else {
					set center1 [list $x $y $z];
					set center2 [list $x2 $y2 $z2];
					set id_graph_prim [::epock::display_maximum_inclusion_volume_cylinder_byFile $center1 $center2 $::epock::molid $::epock::grid_size $::epock::radius "red"]		
				}

				if { [.epock.nb.tab1.tree exists $::epock::cavity_name] == 1} {
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "exclude_cylinder"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center1 0]] [format %0.2f [lindex $center1 1]] [format %0.2f [lindex $center1 2]]   [format %0.2f [lindex $center2 0]] [format %0.2f [lindex $center2 1]] [format %0.2f [lindex $center2 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				} else {
					.epock.nb.tab1.tree insert {} end -id $::epock::cavity_name -text "$::epock::cavity_name"
					.epock.nb.tab1.tree set  $::epock::cavity_name Shape ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Center ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Radius ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Id_primitive ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Molid ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Hidden 0
					.epock.nb.tab1.tree set  $::epock::cavity_name Graph_prim ""

					.epock.nb.tab1.tree set  $id Shape "exclude_cylinder"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center1 0]] [format %0.2f [lindex $center1 1]] [format %0.2f [lindex $center1 2]]   [format %0.2f [lindex $center2 0]] [format %0.2f [lindex $center2 1]] [format %0.2f [lindex $center2 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::radius"   
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim
					.epock.nb.tab1.tree set  $id Molid $::epock::molid
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				}
			}
			
			if {$::epock::choice_method == "box"} {
				if {$byButton} {
					set center [measure center [atomselect $::epock::molid $::epock::target]]

					#create graphical primitive
					# launch the function ::epock::display_maximum_inclusion_volume_box from the epock.tcl file	
					set id_graph_prim [::epock::display_maximum_inclusion_volume_box $::epock::target $::epock::x_box $::epock::y_box $::epock::z_box  $::epock::molid $::epock::grid_size "red"]
				} else {
					set center [list $x $y $z]
					set id_graph_prim [::epock::display_maximum_inclusion_volume_box_byFile $center $::epock::x_box $::epock::y_box $::epock::z_box  $::epock::molid $::epock::grid_size "red"]
				}
				# store infos in the treeview
				## Error here: new cavity name is not popular
				if { [.epock.nb.tab1.tree exists $::epock::cavity_name] == 1} {					
					set id [.epock.nb.tab1.tree insert $::epock::cavity_name 0 -text "$::epock::cavity_name"]
					.epock.nb.tab1.tree set  $id Shape "exclude_box"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::x_box $::epock::y_box $::epock::z_box" 
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim 
					.epock.nb.tab1.tree set  $id Molid $::epock::molid 
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				} else {
					.epock.nb.tab1.tree insert {} end -id $::epock::cavity_name -text "$::epock::cavity_name"
					.epock.nb.tab1.tree set  $::epock::cavity_name Shape ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Center ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Radius ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Id_primitive ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Molid ""
					.epock.nb.tab1.tree set  $::epock::cavity_name Hidden 0
					.epock.nb.tab1.tree set  $::epock::cavity_name Graph_prim ""

					.epock.nb.tab1.tree set  $id Shape "exclude_box"
					.epock.nb.tab1.tree set  $id Center "[format %0.2f [lindex $center 0]] [format %0.2f [lindex $center 1]] [format %0.2f [lindex $center 2]]"
					.epock.nb.tab1.tree set  $id Radius "$::epock::x_box $::epock::y_box $::epock::z_box" 
					.epock.nb.tab1.tree set  $id Id_primitive $id_graph_prim 
					.epock.nb.tab1.tree set  $id Molid $::epock::molid 
					.epock.nb.tab1.tree set  $id Hidden 0
					.epock.nb.tab1.tree set  $id Graph_prim ""
				}
			}
		}
	}
}; # End AddSmth


# ::epock::openCFGfile ------------------------------------------------------
#    Pop-ups the GUI for loading a cfg file. 
#    Parses the file.
#    Then closes the file.
# ------------------------------------------------------------------------------

proc ::epock::openCFGfile {} {
	set myepockFile [tk_getOpenFile -title "Load saved epock configuration file"]
	if { $myepockFile == "" } {
		puts "\nYou selected no file! Try again.\n"
		return;
	} else {
		set datafileID [open $myepockFile r];
		puts "Opened $myepockFile for reading."
		
		set lineN 0
		set parseCavity 0
		set cavityNameHere 0
		set cavityName ""

		
		foreach {i} [split [read $datafileID] \n] {				# Split the file up into single lines

		set is_a_comment_line 0
		set diese "#"
		
			set lineWords [split $i " \n\r\t" ]
				foreach word $lineWords {
				
				 if {$word == $diese} {
					set is_a_comment_line 1
				}
			}	

			if {$is_a_comment_line == 0} {	
				
				if {$lineN == 1} {
					set lineWords [split $i " \n\r\t" ]
					set ::epock::grid_size [lindex $lineWords 2]
				}

				if {$parseCavity && !$cavityNameHere} {
					## proper cavity element
					set spaceEdit [join $i " "]
					set lineWords [split $spaceEdit " \n\r\t" ]
					set in_exclude_shape [split [lindex $lineWords 0] "_" ]
		
					if {[lindex $in_exclude_shape 0] == "include"} {
						set include 1
					} elseif {[lindex $in_exclude_shape 0] == "exclude"} {
						set include 0
					} elseif {[lindex $in_exclude_shape 0] == "contiguous"} {
						set include 1
					} 
					
					#puts "Include = $include"
					set shape_x  [lindex $lineWords 2]
					set shape_y  [lindex $lineWords 3]
					set shape_z  [lindex $lineWords 4]
					
					if {[lindex $in_exclude_shape 1] == "sphere"} {
						
						# change the interface value --- 
						set ::epock::choice_method "sphere"
						set ::epock::choice 1
						
						# enable
						.epock.nb.tab1.radius_msg configure -foreground black
						.epock.nb.tab1.radius configure -state normal
						.epock.nb.tab1.seed configure -state normal

						# disable
						.epock.nb.tab1.x_box_msg configure -foreground "gray60"
						.epock.nb.tab1.x_box configure -state disabled
						.epock.nb.tab1.y_box_msg configure -foreground "gray60"
						.epock.nb.tab1.y_box configure -state disabled
						.epock.nb.tab1.z_box_msg configure -foreground "gray60"
						.epock.nb.tab1.z_box configure -state disabled
						.epock.nb.tab1.target2_msg configure -foreground "gray60"
						.epock.nb.tab1.target2 configure -state disabled
						
						# end the interface value --- 
									
						set ::epock::radius  [lindex $lineWords 5]	
						# Send to production
						#puts "sending a sphere to $::epock::cavity_name !"
						::epock::addSmth $include 0 $shape_x $shape_y $shape_z 0 0 0 0
						
					} elseif {[lindex $in_exclude_shape 1] == "seed"} {
						puts [lindex $in_exclude_shape 2]
						if {[lindex $in_exclude_shape 2] == "sphere"} {
							
							# change the interface value --- 
							set ::epock::choice_method "sphere"
							set ::epock::choice 1
							
							# enable
							.epock.nb.tab1.radius_msg configure -foreground black
							.epock.nb.tab1.radius configure -state normal
							.epock.nb.tab1.seed configure -state normal

							# disable
							.epock.nb.tab1.x_box_msg configure -foreground "gray60"
							.epock.nb.tab1.x_box configure -state disabled
							.epock.nb.tab1.y_box_msg configure -foreground "gray60"
							.epock.nb.tab1.y_box configure -state disabled
							.epock.nb.tab1.z_box_msg configure -foreground "gray60"
							.epock.nb.tab1.z_box configure -state disabled
							.epock.nb.tab1.target2_msg configure -foreground "gray60"
							.epock.nb.tab1.target2 configure -state disabled
							
							# end the interface value --- 
							
							
							set ::epock::radius  [lindex $lineWords 5]	
							# Send to production
							#puts "sending a seed to $::epock::cavity_name !"
							::epock::addSmth $include 0 $shape_x $shape_y $shape_z 0 0 0 1
						} elseif {[lindex $in_exclude_shape 2] == "cylinder"} {
							
							# change the interface value --- 
							set ::epock::choice_method "cylinder"
							set ::epock::choice 3
							# enable
							.epock.nb.tab1.target2_msg configure -foreground black
							.epock.nb.tab1.target2 configure -state normal
							.epock.nb.tab1.radius_msg configure -foreground black
							.epock.nb.tab1.radius configure -state normal
							.epock.nb.tab1.seed configure -state normal

							# disable
							.epock.nb.tab1.x_box_msg configure -foreground "gray60"
							.epock.nb.tab1.x_box configure -state disabled
							.epock.nb.tab1.y_box_msg configure -foreground "gray60"
							.epock.nb.tab1.y_box configure -state disabled
							.epock.nb.tab1.z_box_msg configure -foreground "gray60"
							.epock.nb.tab1.z_box configure -state disabled
	
							# end the interface value --- 
							
							
							set shape_x2  [lindex $lineWords 5]
							set shape_y2  [lindex $lineWords 6]
							set shape_z2  [lindex $lineWords 7]
							set ::epock::radius  [lindex $lineWords 8]	
							
							# Send to production
							#puts "sending a seed to $::epock::cavity_name !"
							::epock::addSmth $include 0 $shape_x $shape_y $shape_z $shape_x2 $shape_y2 $shape_z2 1
						} elseif {[lindex $in_exclude_shape 2] == "box"} {
						
						# change the interface value --- 
						set ::epock::choice_method "box"
						set ::epock::choice 2			
						# enable
						.epock.nb.tab1.x_box_msg configure -foreground black
						.epock.nb.tab1.x_box configure -state normal
						.epock.nb.tab1.y_box_msg configure -foreground black
						.epock.nb.tab1.y_box configure -state normal
						.epock.nb.tab1.z_box_msg configure -foreground black
						.epock.nb.tab1.z_box configure -state normal
						.epock.nb.tab1.seed configure -state normal
						
						# disable
						.epock.nb.tab1.radius_msg configure -foreground "gray60"
						.epock.nb.tab1.radius configure -state disabled
						.epock.nb.tab1.target2_msg configure -foreground "gray60"
						.epock.nb.tab1.target2 configure -state disabled
						# end the interface value --- 
														
							set ::epock::x_box [lindex $lineWords 5]
							set ::epock::y_box [lindex $lineWords 6]
							set ::epock::z_box [lindex $lineWords 7]
						
							# Send to production
							#puts "sending a box to $::epock::cavity_name !"
							::epock::addSmth $include 0 $shape_x $shape_y $shape_z 0 0 0 1
						}	
							
					} elseif {[lindex $in_exclude_shape 1] == "cylinder"} {

						# change the interface value --- 
							set ::epock::choice_method "cylinder"
							set ::epock::choice 3
							# enable
							.epock.nb.tab1.target2_msg configure -foreground black
							.epock.nb.tab1.target2 configure -state normal
							.epock.nb.tab1.radius_msg configure -foreground black
							.epock.nb.tab1.radius configure -state normal
							.epock.nb.tab1.seed configure -state normal

							# disable
							.epock.nb.tab1.x_box_msg configure -foreground "gray60"
							.epock.nb.tab1.x_box configure -state disabled
							.epock.nb.tab1.y_box_msg configure -foreground "gray60"
							.epock.nb.tab1.y_box configure -state disabled
							.epock.nb.tab1.z_box_msg configure -foreground "gray60"
							.epock.nb.tab1.z_box configure -state disabled
	
							# end the interface value ---
							
						set shape_x2  [lindex $lineWords 5]
						set shape_y2  [lindex $lineWords 6]
						set shape_z2  [lindex $lineWords 7]
						
						set ::epock::radius  [lindex $lineWords 8]
						# Send to production
						#puts "sending a cylinder to $::epock::cavity_name !"
						::epock::addSmth $include 0 $shape_x $shape_y $shape_z $shape_x2 $shape_y2 $shape_z2 0
							
					} elseif {[lindex $in_exclude_shape 1] == "box"} {
						
						# change the interface value --- 
						set ::epock::choice_method "box"
						set ::epock::choice 2			
						# enable
						.epock.nb.tab1.x_box_msg configure -foreground black
						.epock.nb.tab1.x_box configure -state normal
						.epock.nb.tab1.y_box_msg configure -foreground black
						.epock.nb.tab1.y_box configure -state normal
						.epock.nb.tab1.z_box_msg configure -foreground black
						.epock.nb.tab1.z_box configure -state normal
						.epock.nb.tab1.seed configure -state normal
						
						# disable
						.epock.nb.tab1.radius_msg configure -foreground "gray60"
						.epock.nb.tab1.radius configure -state disabled
						.epock.nb.tab1.target2_msg configure -foreground "gray60"
						.epock.nb.tab1.target2 configure -state disabled
						# end the interface value --- 
						
											
						set ::epock::x_box [lindex $lineWords 5]
						set ::epock::y_box [lindex $lineWords 6]
						set ::epock::z_box [lindex $lineWords 7]
						
						# Send to production
						#puts "sending a box to $::epock::cavity_name !"
						::epock::addSmth $include 0 $shape_x $shape_y $shape_z 0 0 0 0
					}
				}
				
				if {$cavityNameHere} {
					## Cavity name
					set ::epock::cavity_name [string range $i 1 end-1]
					set cavityNameHere 0
				}
				
				if {$i == ""} {																			;# Counting on the first space
					set parseCavity 1
					set cavityNameHere 1
				}
				set lineN [expr $lineN + 1]
		}
	}
		close $datafileID;
	}
}

# ::epock::hide_graphic_primitive ------------------------------------------
#    hide/show graphics primitive selected. 
# ------------------------------------------------------------------------------
proc ::epock::hide_graphic_primitive {graph_id molid hidden_tag cavity_selection} {


			if {$hidden_tag == 0} {
				
				set list_graphics ""
				set list_element_id ""
				set color_graphics 0
				set shape_name  [lindex [ .epock.nb.tab1.tree item  $cavity_selection -values] 0]
				if {[string index $shape_name 0] == "i"} {
					set color_graphics 4
				} elseif {[string index $shape_name 0] == "e"} {
					set color_graphics 1
				} elseif {[string index $shape_name 0] == "c"} {
					set color_graphics 10				
				}
				
				foreach element_id $graph_id {
					set graph_prim_hidden [graphics $molid info $element_id]
					lappend list_graphics "$graph_prim_hidden"
					lappend list_element_id $element_id

					graphics $molid replace $element_id
					graphics $molid point "0 0 0"

				}
				set list_graph_prim ""
				lappend list_graph_prim $list_element_id
				lappend list_graph_prim $list_graphics
				lappend list_graph_prim $color_graphics

				.epock.nb.tab1.tree set  $cavity_selection Graph_prim $list_graph_prim
			} 
			if {$hidden_tag == 1} {

				set list_graphic_shown [lindex [ .epock.nb.tab1.tree item  $cavity_selection -values] 6]	

				graphics $molid color [lindex $list_graphic_shown 2]
				for {set i 0} {$i <[llength [lindex $list_graphic_shown 0]] } {incr i} {	
					graphics $molid replace [lindex [lindex $list_graphic_shown 0] $i]
					set graphic_primitive [lindex [lindex $list_graphic_shown 1] $i]
					#puts $graphic_primitive	
					eval [concat "graphics $molid" $graphic_primitive]			
				}
			}
}


proc ::epock::quit {} {
	puts "Info) Quitting epock. Adieu!"
	::epock::reset
	wm withdraw .epock
}

# TO DO to clean the variables correctly when closing the window
proc ::epock::reset {} {
	
	#set ::epock::version 1.0
	#set ::epock::realtime 1
	#set ::epock::choice1 0
	#set ::epock::choice2 1
	#set ::epock::choice2_param1 0
	#set ::epock::choice2_param2 0
	#set ::epock::choice2_param3 0
	#set ::epock::choice3_param1 "CA"
	#set ::epock::target "resid 12 to 300"
	
	#.epock.nb.tab1.target delete 0.0 end
	#.epock.nb.tab1.target insert 1.0 $::epock::target
	#.epock.nb.tab1.target2 delete 0.0 end
	#.epock.nb.tab1.target2 insert 1.0 $::epock::target2
	.epock.nb.tab1.x_box_msg configure -foreground "gray60"
	.epock.nb.tab1.x_box configure -state disabled
	.epock.nb.tab1.y_box_msg configure -foreground "gray60"
	.epock.nb.tab1.y_box configure -state disabled
	.epock.nb.tab1.z_box_msg configure -foreground "gray60"
	.epock.nb.tab1.z_box configure -state disabled
}

proc epock_tk {} {
	::epock::epockGUI
	return $::epock::dw
}

